#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <EEPROM.h>
#include <ESP8266HTTPClient.h>
#include "WEBset.h"  // Hier ist die HTML-Webseite als Array abgelegt

#define LEDpin 2   // LED zum Blinken. Bei ESP-07 Pin 2. Bei ESP-01 Pin 1
#define Sensor_PIN 5 // Pin der an Reedkontackt angeschlossen ist
#define DNS
// #define seriell // Einschalter für serielle Ausgabe 

IPAddress apIP(192, 168, 168, 30);  // IP-Adresse für Setup

#if defined DNS               // Fuer Setupmodus
#include <DNSServer.h>
const byte DNS_PORT = 53;
DNSServer dnsServer;
#endif

ADC_MODE (ADC_VCC);           // ADC an das Stromversorgung
uint ADCKal34, ADCKal24;      // Variablen für Kalibrationswerte
int z = 0;                    //Aktuelle EEPROM-Adresse zum lesen
String nachricht = "Testnachricht";
String url = "";
int timout = 0;
char ssid[32] = "fhgdhfghd\0";
char passwort[64] = "gghdfgh\0";

ESP8266WebServer server;
EspClass ESPm;
HTTPClient http;
#include "Setup.h"

extern "C" {
#include "user_interface.h"
}

void Fehlgeschlagen()
{
  http.end();
#if defined seriell       // Entweder Serielle Meldungen
  Serial.println("Verbindung zum Server fehlgeschlagen");
  delay(100);
#else
  digitalWrite(LEDpin, 0);
  delay(5000);                   //LED für 5Sek ON
  digitalWrite(LEDpin, 1);
#endif
  ESPm.deepSleep(0, WAKE_RFCAL); //gehe schlafen
  delay(100);
}

void setup() {
  char inser;
  pinMode(Sensor_PIN, INPUT);

  EEPROM.begin(512);  // EEPROM initialisieren mit 512 Byts

  //------------------------------------ Prüfen ob Tx und Rx verbunden sind -------------------------------
  pinMode(1, OUTPUT); // Tx
  pinMode(3, INPUT_PULLUP); // Rx
  digitalWrite(1, 1);
  delayMicroseconds(2);
  if (!digitalRead(3))goto weiter;  // Prüfen ob Tx und Rx verbunden sind

  digitalWrite(1, 0);
  delayMicroseconds(2);
  if (digitalRead(3))goto weiter;
  pinMode(LEDpin, OUTPUT);
  digitalWrite(LEDpin, 0);
  Einstellen();                      // TxD ist mit RxD verbunden Setupmodus (Setup.h)
weiter:

#if defined seriell       // Entweder Serielle Meldungen
  Serial.begin(115200);     // oder
#else                     // LED Anzeige
  pinMode(LEDpin, OUTPUT); // Vorbereiten der LED
#endif

  //------------------------------- Einstellungen aus EEPROM einlesen -------------------------------------------
  z = 0;
  LeseEeprom(ssid, sizeof(ssid));
  if (ssid[0] == 255)             // Wenn EEPROM leer ist dann:
  {
    for (int i = 0; i < 6; i++) EEPROM.write(i, '\0'); // mit "0" fullen
    EEPROM.commit();
    digitalWrite(LEDpin, 0);
    Einstellen();               // und in Setup modus wechseln
  }

  LeseEeprom(passwort, sizeof(passwort)); // Rest aus EEPROM einlesen
  LeseEepromStr(&url, 100);
  LeseEepromStr(&nachricht, 100);

  //---------------------------------- ADC - Kalibrationswerte einlesen -------------------------------------------
  char *zeichen;
  zeichen = reinterpret_cast<char*>(&ADCKal34);
  *zeichen = EEPROM.read(240);
  zeichen++;
  *zeichen = EEPROM.read(241);
  zeichen = reinterpret_cast<char*>(&ADCKal24);
  *zeichen = EEPROM.read(242);
  zeichen++;
  *zeichen = EEPROM.read(243);
  if (ADCKal34 > 5000 || ADCKal34 < 2000)ADCKal34 = 3400;
  if (ADCKal24 > 5000 || ADCKal24 < 1500)ADCKal24 = 2400;

  //------------------------------------ ADC Wert korriegieren -------------------------------------------
  uint ADCValue = 0;            // Batteriespannung messen
  ADCValue = ESPm.getVcc();
  unsigned int faktor = ADCKal34 - ADCKal24;
  int Korrekturwert = 2.4 * faktor - ADCKal24;
  float ADCfloat = float(ADCValue + Korrekturwert);
  ADCfloat = ADCfloat / faktor;
  String Ubatt = String(ADCfloat, 2);

  //------------------------------------ Im Station Modus mit Netzwerk verbinden -------------------------------------------
  WiFi.mode(WIFI_STA);

# if defined  seriell              // Serielle Ausgaben falls freigeschaltet
  Serial.println();
  Serial.print("U Batt:  ");
  Serial.println(Ubatt);
  Serial.print("Connecting to ");
  Serial.println (ssid);
  Serial.println (url);
  Serial.println (nachricht);
#endif

  WiFi.begin(ssid, passwort);

  while (WiFi.status() != WL_CONNECTED)     // Solange nicht verbunden versuche 6S zu verbinden
  {

    //----------------------------------------------- Timeout abgelaufen ---------------------------------------
    if  (timout > 60)         // Wenn Anmeldung nicht möglich
    {
#if defined seriell
      Serial.print("Netzwerk nicht gefunden");
#else
      pinMode(LEDpin, OUTPUT);
      digitalWrite(LEDpin, 0);  // LED ON für 5 Sekunden
      delay(5000);
      digitalWrite(LEDpin, 1);
#endif
      ESPm.deepSleep(0, WAKE_RFCAL); // Sleep um Batterie zu sparen
      delay(100);
    }                           // End timeout

    //------------------------------- Anzeige Während Verbindungsaufbaus -------------------------------------------------
    timout++;
#if defined seriell
    delay(500);
    Serial.print("O");
#else
    digitalWrite(LEDpin, 0);
    delay(50);
    digitalWrite(LEDpin, 1);
    delay(50);
#endif
  }//--------------------------------- Verbunden -------------------------------------------------------------------------

url = url + "field1=" + String(digitalRead(Sensor_PIN))+"&"; 
  url = url + "field2=" + Ubatt;      //Parameter Bat Spannung
  //  url = url + "&Nachricht=" + urlencode(nachricht) + String(Zustand);

  http.begin(url); //HTTP
  int httpCode = http.GET();

  if (httpCode > 0) // Wenn Serverantwort erhalten
  {
# if defined  seriell
    Serial.printf("[HTTP] GET... code: %d\n", httpCode);
#endif
    if (httpCode == HTTP_CODE_OK)  // Wenn Webseite gefunden
    {
      String payload = http.getString();

# if defined  seriell
      Serial.print("Serverantwort: ");
      Serial.println(payload);
      delay(100);
#endif
      http.end();
      ESPm.deepSleep(0, WAKE_RFCAL); //gehe schlafen
      delay(100);
    }  // End Websete gefunden
    else
    { // Websete nicht gefunden
# if defined  seriell
      Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
#endif
      Fehlgeschlagen();
    }// End Websete nicht gefunden
  } // End Wenn Serverantwort erhalten
  Fehlgeschlagen();
}

void loop()
{
  yield();
}

